/* Copyright (C) 2009 Papavasileiou Dimitris                             
 *                                                                      
 * This program is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or    
 * (at your option) any later version.                                  
 *                                                                      
 * This program is distributed in the hope that it will be useful,      
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 * GNU General Public License for more details.                         
 *                                                                      
 * You should have received a copy of the GNU General Public License    
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <lua.h>
#include <lauxlib.h>
#include <GL/gl.h>

#include "row.h"

@implementation Row

-(void) transform
{
    id child;
    float width;

    /* Calculate our own allocation based on our children. */
    
    self->minimum[0] = self->padding[0];
    self->minimum[1] = 2 * self->padding[1];
    
    for(child = [self children] ; child ; child = [child sister]) {
	float height;

	height = [child height] + 2 * self->padding[1];
	
	self->minimum[0] += [child width] + self->padding[0];
	self->minimum[1] = height > self->minimum[1] ?
	                   height : self->minimum[1];
    }

    self->allocated[0] = self->minimum[0] > self->requested[0] ?
	                 self->minimum[0] : self->requested[0];
    self->allocated[1] = self->minimum[1] > self->requested[1] ?
	                 self->minimum[1] : self->requested[1];

    /* Align ourselves. */
    
    if (self->align[0] < 0) {
	self->position[0] += 0.5 * (self->minimum[0] - self->allocated[0]);
    } else if (self->align[0] > 0) {
	self->position[0] += 0.5 * (self->allocated[0] - self->minimum[0]);
    }

    if (self->align[1] < 0) {
	self->position[1] += 0.5 * (self->minimum[1] - self->allocated[1]);
    } else if (self->align[1] > 0) {
	self->position[1] += 0.5 * (self->allocated[1] - self->minimum[1]);
    }
	
    /* Now distribute the children in the allocated space. */
    
    for(child = [self children],
	width = self->padding[0] - 0.5 * (self->minimum[0] - [child width]);
	child ;
	width += 0.5 * ([child width] + [[child sister] width]) +
	          self->padding[0],
	child = [child sister]) {
	
	GLfloat delta[3] = {width, 0, 0};

	[child setPositionTo: delta];
    }

    [super transform];
}

@end

